/*
 * checkerberry test center - https://www.checkerberry.de
 *
 * Copyright (c) 2010 ConceptPeople consulting gmbh
 *
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/software/page/eupl5
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.conceptpeople.checkerberry.db.example;

import de.conceptpeople.checkerberry.common.validation.ValidationStatus;
import de.conceptpeople.checkerberry.common.validation.Validator;
import de.conceptpeople.checkerberry.common.validation.ValidatorId;

/**
 * Definiert einen eigenen Validator für die Spalte WEIGHT. Das Gewicht wird in
 * BMI-Klassen eingeteilt und dann anhand der Klasse validiert. Die Klassen sind
 * <ul>
 * <li>&gt; 70 kg: BMI bad</li>
 * <li>&gt; 50 kg: BMI nice</li>
 * <li>&gt; 35 kg: BMI Grab some food!</li>
 * <li>&lt;= 35 kg: BMI -</li>
 * </ul>
 * .
 * 
 * In den erwarteten Testdaten kann in der Spalte WEIGHT z.B. "BMI bad"
 * angegeben werden. Der Validator prüft dann, ob der tatsächliche Wert > 70.0
 * ist. Ist dies nicht der Fall, wird ein Validierungsfehler geworfen.
 * <p/>
 * Die Sinnhaftigkeit dieses Validators darf angezweifelt werden, für
 * Demo-Zwecke sollte er jedoch ok sein.
 */
public class WeightValidator implements Validator {

    /** Id des Validators sollte immer public static sein. */
    public static final ValidatorId ID = new ValidatorId("WeightValidator");

    /**
     * {@inheritDoc}
     */
    @Override
    public ValidationStatus validate(String currentValueAsString, String expectedValueWithOperatorAsString) {

        // Konvertiert den tatsächlichen Wert in einen Double. (Würde man im
        // normalen Leben wegen der Rundungsfehler schon nicht machen.
        // BigDecimal ist Dein Freund...)
        double weight;
        try {
            weight = Double.parseDouble(currentValueAsString);
        } catch (NumberFormatException e) {
            // Wenn wir das Gewicht nicht in einen Double konvertieren können,
            // ist dieser Validator vermutlich doch nicht für die Validierung
            // dieses Feldes zuständig. Die Validierung wird an den
            // nachfolgenden Validator delegiert.
            return ValidationStatus.VALIDATION_SKIPPED;
        }

        // Anhand des tatsächlichen Gewichts, können wir bestimmen, welche
        // Klasse wir erwarten würden. Den Wert bauen wir hier zusammen.
        String expectedValue = "BMI ";
        if (weight > 70.0) {
            expectedValue += "bad";
        } else if (weight > 50.0) {
            expectedValue += "nice";
        } else if (weight > 35.0) {
            expectedValue += "Grab some food!";
        } else {
            expectedValue += "-";
        }

        // Wenn der erwartete Wert mit dem konstruierten Wert übereinstimmt, war
        // die Validierung erfolgreich. Nachfolgende Validatoren werden dann
        // nicht mehr aufgerufen.
        if (expectedValue.equalsIgnoreCase(expectedValueWithOperatorAsString)) {
            return ValidationStatus.VALIDATED_WITH_SUCCESS;
        }

        // Ansonsten ist der Wert wohl unterschiedlich, sodass die Validierung
        // fehlschlägt :-(
        return ValidationStatus.VALIDATED_WITH_FAILURE;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean matches(String expectedValue) {

        // Dieser Validator fühlt sich nur verantwortlich, wenn der erwartete
        // Wert vorhanden ist und mit "BMI " anfängt. Alle anderen Werte
        // ignoriert er.
        return expectedValue != null && expectedValue.startsWith("BMI");
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public ValidatorId getValidatorId() {
        return ID;
    }

}
