/*
 * checkerberry test center - https://www.checkerberry.de
 *
 * Copyright (c) 2010 ConceptPeople consulting gmbh
 *
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/software/page/eupl5
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.conceptpeople.checkerberry.db.example.autoparameter;

import java.util.ArrayList;

import org.junit.Assert;
import org.junit.Test;

import de.conceptpeople.checkerberry.db.example.AbstractExampleDaoTestCase;
import de.conceptpeople.checkerberry.db.example.dao.UserDao;
import de.conceptpeople.checkerberry.db.example.entities.Dish;
import de.conceptpeople.checkerberry.db.example.entities.User;

/**
 * Dieser Beispieltest enthält die Verwendung der Autoparameter.
 */
public class DbExampleAutoParameterIntegrationTest extends AbstractExampleDaoTestCase {

    /**
     * Testet die Methode {@link UserDao#save(User)}. In diesem Fall ein
     * Insert-Statement, das auch eine Zuordnungstabelle anlegt.
     *
     * @throws Exception
     *             wenn ein Fehler aufgetreten ist.
     */
    @Test
    public void testInsertUser() throws Exception {

        // Die Testdaten unter
        // DbExampleAutoParameterIntegrationTest_testInsertUser_initial.xml
        // wurden in der Setup-Phase schon eingespielt, sodass es hier gleich
        // losgehen kann.

        // Anlegen einer Liste von Mahlzeiten, die zu Homer Simpson gespeichert
        // werden sollen.
        ArrayList<Dish> dishes = new ArrayList<Dish>();
        dishes.add(new Dish("Donuts"));
        dishes.add(new Dish("Duff is a dish"));

        getDishDao().save(dishes.get(0));
        getDishDao().save(dishes.get(1));

        // Neues User-Objekt für Homer anlegen...
        User homer = new User("Homer", "Simpson", createDate(16, 9, 1946));
        homer.setWeight(80.0);
        // ...und seine Lieblingsspeisen zuordnen.
        homer.setDishes(dishes);

        getUserDao().save(homer);

        // Prüfen der Ergebnisse in
        // DbExampleAutoParameterIntegrationTest_testInsertUser_result.xml. Die
        // korrekte Zuordnung der User- und Dish-Objekte wird über Autoparameter
        // direkt in den Testdaten sichergestellt.
        getTestHandler().assertEqualsExpected();
    }

    /**
     * Demonstriert den Fall, dass nur ein Autoparameter verwendet wird.
     *
     * @throws Exception
     *             wenn ein Fehler aufgetreten ist.
     */
    @Test
    public void testSingleAutoParameter() throws Exception {

        // Die Testdaten unter
        // DbExampleAutoParameterIntegrationTest_testSingleAutoParameter_initial.xml
        // wurden in der Setup-Phase schon eingespielt, sodass es hier gleich
        // losgehen kann.

        // Neues User-Objekt für Homer anlegen...
        User homer = new User("Homer", "Simpson", createDate(16, 9, 1946));
        homer.setWeight(80.0);

        getUserDao().save(homer);

        // Prüfen der erwarteten Testdaten unter
        // DbExampleAutoParameterIntegrationTest_testSingleAutoParameter_result.xml.
        // Dabei werden alle Autoparameter ermittelt, ...
        getTestHandler().assertEqualsExpected();

        // ...die danach ausgelesen werden können.
        String homerWeight = getTestHandler().getAutoParameterContext().getAutoParameterValue("homer.weight");

        // Durch die Verwendung eines einzelnen Autoparameters wurde der Wert
        // von der Prüfung ausgeschlossen. Dies ist somit eine Möglichkeit,
        // einzelne Werte von der Prüfung auszuschließen ohne gleich die gesamte
        // Spalte über excluded columns auszuschließen.

        // Außerdem könnte man jetzt im Test besondere Prüfungen machen. Hier
        // ist allerdings zu prüfen, ob das Schreiben eines eigenen Validators
        // nicht sinnvoller ist. Die Verwendung eines eigenen Validators ist
        // dann sinnvoller, wenn die Prüfung immer auf bestimmte Spalten
        // ausgeführt werden soll.
        Assert.assertTrue(Double.parseDouble(homerWeight) > 75.0);
    }
}
