/*
 * checkerberry test center - https://www.checkerberry.de
 *
 * Copyright (c) 2010 ConceptPeople consulting gmbh
 *
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/software/page/eupl5
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.conceptpeople.checkerberry.db.example.cache;

import java.util.List;

import org.junit.Assert;
import org.junit.FixMethodOrder;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.junit.runners.MethodSorters;
import org.springframework.test.context.junit4.SpringJUnit4ClassRunner;

import de.conceptpeople.checkerberry.db.core.test.ClearTables;
import de.conceptpeople.checkerberry.db.core.test.ClearTables.TableTypes;
import de.conceptpeople.checkerberry.db.core.test.NoCache;
import de.conceptpeople.checkerberry.db.example.AbstractExampleDaoTestCase;
import de.conceptpeople.checkerberry.db.example.entities.Dish;

/**
 * Beispieltest für die Verwendung des Tabellencaches.
 * <p/>
 * Der Tabellencache lässt sich relativ schlecht in Form einer Beispielklasse
 * beschreiben, da er über alle Tests hinweg verwendet wird. Außerdem ist er von
 * der Ausführungsreihenfolge der Testmethoden abhängig, sodass bei einer
 * geänderten Ausführungsreihenfolge die demonstrierten Eigenschaften ggf. nicht
 * sichtbar sind. In dieser Beispielklasse gehen wir davon aus, dass die Tests
 * in der Reihenfolge test1xxx, bis, testnxxx ausgeführt werden, was durch die
 * Annotation @FixMethodOrder mit dem Paramter MethodSorters.NAME_ASCENDING
 * erreicht wird.
 * <p/>
 * Bitte schauen Sie sich auch das entsprechende Kapitel im Benutzerhandbuch an.
 * Dort wird das ansich einfache Verfahren anhand von Grafiken anschaulich
 * erläutert.
 */
@RunWith(SpringJUnit4ClassRunner.class)
@FixMethodOrder(MethodSorters.NAME_ASCENDING)
public class DbExampleCacheIntegrationTest extends AbstractExampleDaoTestCase {

    /**
     * In unserem Beispiel wird die Tabelle DISH als Cacheable markiert.
     * Fachlich könnte man es uzngefähr so erklären: Ein Restaurant führt eine
     * Liste der Kunden und ihrer Lieblingsspeisen. Die Speisekarte ist fest,
     * d.h. die Tabelle DISH enthällt immer die gleichen Gerichte. Damit die
     * Daten nicht bei jedem Test neu eingespielt werden müssen, werden sie als
     * cacheable markiert, nur einmal eingespielt und dann von allen Tests
     * wiederverwendet.
     * <p/>
     * In dieser Methode wird der Cache aufgebaut.
     */
    @Test
    public void test1CreateDishCache() {
        // Prüft, ob die Datenbank die Standard-Speisekarte enthält.
        assertContainsCommonDishes();

        // Außerdem soll Homer über die initialen Daten in die Datenbank
        // eingespielt worden sein.
        assertContainsOnlyUser("Homer");
    }

    /**
     * In dieser Methode zeigen wir, dass der Cache funktioniert und die
     * Einträge der Tabelle DISH noch vorhanden sind.
     */
    @Test
    public void test2UseDishCache() {
        // Prüft, ob die Datenbank die Standard-Speisekarte enthält. Zu beachten
        // ist, dass die initialen Testdaten gar keine Einträge der Tabelle DISH
        // beinhalten. Das ist in der Regel ein Fehler, wird hier jedoch
        // genutzt, um zu demonstrieren, dass das Caching funktioniert.
        assertContainsCommonDishes();

        // Außerdem soll Marge über die initialen Daten in die Datenbank
        // eingespielt worden sein.
        assertContainsOnlyUser("Marge");
    }

    /**
     * In dieser Methode zeigen wir einen Stolperstein, der zu vermeiden ist: In
     * den initialen Testdaten werden vom Cache abweichende Werte für die
     * Tabelle DISH verwendet. Da aber bereits dem Cache vermerkt ist, dass die
     * Tabelle DISH eingespielt wurde, werden die abweichenden Werte nicht
     * eingespielt.<br/>
     * Um diese Situation zu verhinden, sollten für alle Daten, die gecacht
     * werden sollen, eigene Include-Dateien angelegt werden. Diese
     * Include-Dateien sollten alle Testmethoden verwenden, die den
     * entsprechenden Inhalt im Cache erwarten. Auf diese Art und Weise wird die
     * Wartung der Testdaten deutlich vereinfacht. Wenn ein Test auf andere
     * Werte für die gecachten Tabellen zugreifen muss, muss die Annotation
     * {@link NoCache} verwendet werden.
     */
    @Test
    public void test3InvalidUseDishCache() {
        // Prüft, ob die Datenbank die Standard-Speisekarte enthält. Zu beachten
        // ist, dass die initialen Testdaten andere Einträge für die Tabelle
        // DISH beinhalten. Das ist in der Regel ein Fehler, wird hier jedoch
        // genutzt, um zu demonstrieren, dass das Caching funktioniert.
        assertContainsCommonDishes();

        // Außerdem soll Bart über die initialen Daten in die Datenbank
        // eingespielt worden sein.
        assertContainsOnlyUser("Bart");
    }

    /**
     * In dieser Methode brauchen wir spezielle Einträge in der Tabelle DISH,
     * die sich von den Standard-Werten unterscheiden. Aus diesem Grund wird der
     * Cache deaktiviert.
     * <p/>
     * Auf diese Art und Weise haben Sie die Möglichkeit Services ode DAO für
     * Tabellen zu testen, die Sie als cacheable markiert haben.
     * <p/>
     * Man kann in der Annotation auch explizit angeben, für welche Tabellen der
     * Cache deaktiviert werden soll. Ohne die Angabe von Tabellennamen wird der
     * gesamte Cache deaktiviert.
     */
    @Test
    @NoCache
    public void test4NoDishCache() {
        // Prüft, ob die Datenbank die neuen Einträge der Tabelle DISH
        // beinhaltet. Der Cache wurde für diesen Test deaktiviert, sodass die
        // Einträge aus den initialen Daten eingespielt werden.
        List<Dish> dish = getDishDao().getDish();
        Assert.assertEquals(2, dish.size());
        Assert.assertTrue(dish.get(0).getDescription().startsWith("Special Menu"));
        Assert.assertTrue(dish.get(1).getDescription().startsWith("Special Menu"));

        // Außerdem soll Lisa über die initialen Daten in die Datenbank
        // eingespielt worden sein.
        assertContainsOnlyUser("Lisa");
    }

    /**
     * In dieser Methode sieht man, dass der Cache für die Tabelle DISH
     * tatsächlich deaktiviert wurde.
     */
    @Test
    public void test5DishCacheEmpty() {
        // Prüft, ob die Tabelle DISH leer ist, wenn sie nicht mehr in den
        // initialen Daten vorhanden ist. Das sollte so sein, da der Cache durch
        // die vorherige Methode geleert wurde.
        List<Dish> dish = getDishDao().getDish();
        Assert.assertEquals(0, dish.size());

        // Außerdem soll Maggie über die initialen Daten in die Datenbank
        // eingespielt worden sein.
        assertContainsOnlyUser("Maggie");
    }

    /**
     * In dieser Methode wird der Cache wieder gefüllt.
     * <p/>
     * An diesem Beispiel können Sie gut erkennen, wie sich der Cache während
     * der gesamten Testausführung verhält. Er wird aufgebaut, wiederverwendet,
     * ggf. zwischenzeitlich gelöscht, wieder neu aufgebaut und wiederverwendet.
     * Gerade bei einer großen Anzahl gecachter Tabelleneinträge hat das Caching
     * großen Einfluss auf die Performance.
     * <p/>
     * Auf der anderen Seite können fehlerhafte Testdaten in Kombination mit
     * eingeschaltetem Caching zu verwirrenden Ergebnissen führen. Das liegt
     * daran, dass das Caching Auswirkungen auf die Reihenfolge der
     * Testausführung hat. In dieser Klasse würde es z.B. zu Problemen kommen,
     * wenn die Methode {@link #test2UseDishCache()} als erstes ausgeführt wird.
     * Da in den initialen Testdaten die Angaben der Tabelle DISH fehlen, würde
     * der Test fehlschlagen. Wenn der Test hinter
     * {@link #test1CreateDishCache()} ausgeführt wird, ist er jedoch
     * erfolgreich. Für die Analyse derartiger Probleme, kann der Cache global
     * deaktiviert werden (
     * {@link de.conceptpeople.checkerberry.db.bridge.configuration.DbConfiguration#setGlobalCacheDisabled(boolean)}
     * ).
     */
    @Test
    public void test6RebuildDishCache() {
        // Prüft, ob die Datenbank wieder die Standard-Speisekarte enthält.
        assertContainsCommonDishes();

        // Außerdem soll Homer über die initialen Daten in die Datenbank
        // eingespielt worden sein.
        assertContainsOnlyUser("Abe");
    }

    /**
     * Mit der Annotation {@link ClearTables} kann der Inhalt aller Tabellen
     * gelöscht werden. Es ist somit eine einfache Möglichkeit, Tests auf einer
     * leeren Datenbank druchzuführen. Wenn die Annotation ohne Parameter
     * aufgerufen wird, werden alle Tabelle, insbesondere auch die gecachten,
     * geleert. Bei der Verwendung des Parameters
     * {@link TableTypes#NON_CACHEABLE} werden nur die nicht gecachten Tabellen
     * gelöscht. Das ist in der Regel ausreichend, da nur Stammdaten gecacht
     * werden, die auf viele Tests keine direkte Auswirkung haben.
     */
    @Test
    @ClearTables(TableTypes.NON_CACHEABLE)
    public void test7ClearNonCacheableTables() {
        // Prüft, ob die Datenbank die Standard-Speisekarte enthält, obwohl
        // diese Methode keine initialen Daten enthält.
        assertContainsCommonDishes();

        // Außerdem dürfen keine User in der Datenbank stehen.
        Assert.assertEquals(0, getUserDao().getUsers().size());
    }

    /**
     * Hier nochmal die Variante, alle Tabelleninhalte über die Annotation
     * {@link ClearTables} zu löschen.
     */
    @Test
    @ClearTables()
    public void test8ClearAllTables() {
        // Die Tabellen DISH und USERS sind beide leer.
        Assert.assertEquals(0, getDishDao().getDish().size());
        Assert.assertEquals(0, getUserDao().getUsers().size());
    }

    /**
     * Prüft, dass nur einer der Simpsons in der Datenbank vorhanden ist.
     *
     * @param firstname
     *            Vorname des erwarteten Simpsons in der Datenbank.
     */
    private void assertContainsOnlyUser(String firstname) {
        Assert.assertEquals(1, getUserDao().getUsers().size());
        Assert.assertNotNull(getUserDao().getUser(firstname, "Simpson"));
    }

    /**
     * Prüft, ob die Datenbank die Standard-Speisekarte enthält.
     */
    private void assertContainsCommonDishes() {
        List<Dish> dishes = getDishDao().getDish();
        Assert.assertEquals(5, dishes.size());

        for (int i = 0; i < dishes.size(); i++) {
            Assert.assertTrue(dishes.get(i).getDescription().startsWith("Standard Menu"));
        }
    }

    /**
     * {@inheritDoc}
     */
    @Override
    protected String getBridgeContainerBeanName() {

        // Speziellen Bridge-Container verwenden, der die Tabelle DISH als
        // Cacheable markiert.
        return "bridgeContainerWithCaching";
    }
}
