/*
 * checkerberry test center - https://www.checkerberry.de
 *
 * Copyright (c) 2010 ConceptPeople consulting gmbh
 *
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/software/page/eupl5
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.conceptpeople.checkerberry.db.example.function;

import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;

import org.junit.Assert;
import org.junit.Test;

import de.conceptpeople.checkerberry.db.example.AbstractExampleDaoTestCase;
import de.conceptpeople.checkerberry.db.example.ExampleConfigurationCallback;
import de.conceptpeople.checkerberry.db.example.WeighingFunction;
import de.conceptpeople.checkerberry.db.example.entities.User;

/**
 * Beispieltest für die Verwendung von Funktionen.
 */
public class DbExampleFunctionIntegrationTest extends AbstractExampleDaoTestCase {

    /**
     * Zeigt die Verwendung der Funktion today mit Offsets.
     */
    @Test
    public void testGetUser() {
        User homer = getUserDao().getUser("Homer", "Simpson");

        Assert.assertNotNull(homer);

        // In der Datenbank ist das Geburtsdatum relativ zu heute angegeben: -46
        // Jahre und +2 Monate.
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(new Date());
        calendar.add(Calendar.YEAR, -46);
        calendar.add(Calendar.MONTH, 2);

        SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd");
        Assert.assertEquals(dateFormat.format(calendar.getTime()), dateFormat.format(homer.getBirthDate()));
    }

    /**
     * Zeigt die Verwendung der Funktion today in den Testdaten.
     *
     * @throws Exception
     *             wenn ein Fehler aufgetreten ist.
     */
    @Test
    public void testInsertUser() throws Exception {
        // Die Testdaten unter
        // DbExampleFunctionIntegrationTest_testInsertUser_initial.xml wurden in
        // der Setup-Phase schon eingespielt, sodass es hier gleich losgehen
        // kann.

        // Der Geburtstag von Homer wird auf das aktuelle Datum gesetzt, d.h.
        // der Wert in der Datenbank ist abhängig vom Zeitpunkt der
        // Testausführung. Damit ist es nicht möglich, einen statischen Wert in
        // den Testdaten zu verwenden. Man könnte einen Parameter verwenden.
        // Einfacher ist hier jedoch die Verwendung der Funktion today, die in
        // den Testdaten verwendet wird.
        User homer = new User("Homer", "Simpson", new Date());
        homer.setWeight(80.0);

        getUserDao().save(homer);

        // Prüfen, ob die Datenbank jetzt den neuen User mit dem korrekten
        // Geburtsdatum enthält.
        getTestHandler().assertEqualsExpected();

        // Mit den Funktionen now und today kann das aktuelle Datum bzw. der
        // aktuelle Zeitpunkt in den Testdaten verwendet werden. Als Parameter
        // können Offsets wie z.B. +2 days oder -1 year verwendet werden.
        // Generell ist es aber immer besser, wenn die zu testende Anwendung die
        // Möglichkeit bietet, die Zeit zu setzen. Nur so kann man wirklich
        // sicher testen, wie sich die Anwendung zu bestimmten Zeitpunkten
        // verhält z.B. Jahreswechsel, Jahrzentwechsel, Schaltjahre o.ä.
    }

    /**
     * Zeigt die Verwendung der benutzerspezifischen Funktion
     * {@link WeighingFunction}, die in {@link ExampleConfigurationCallback}
     * registriert wird.
     *
     * @throws Exception
     *             wenn ein Fehler aufgetreten ist.
     */
    @Test
    public void testCustomFunction() throws Exception {

        // Die Initialdaten
        // DbExampleFunctionIntegrationTest_testCustomFunction_initial.xml
        // enthalten die Gewichte als statische Werte. Die erwarteten Testdaten
        // DbExampleFunctionIntegrationTest_testCustomFunction_result.xml
        // enthält die Gewichte, die über die neue Funktion ermittelt wurden.

        getTestHandler().assertEqualsExpected();
    }

}
