/*
 * checkerberry test center - https://www.checkerberry.de
 *
 * Copyright (c) 2010 ConceptPeople consulting gmbh
 *
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/software/page/eupl5
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.conceptpeople.checkerberry.db.example.include;

import org.junit.Test;

import de.conceptpeople.checkerberry.db.bridge.configuration.DbConfiguration;
import de.conceptpeople.checkerberry.db.bridge.configuration.DbConfigurationCallback;
import de.conceptpeople.checkerberry.db.example.AbstractExampleDaoTestCase;

/**
 * Beispieltest für die Verwendung von inkludierten Testdaten. Testdaten können
 * über das Tag INCLUDE andere Testdaten referenzieren. Sollte in der
 * entsprechenden Datenbank eine Tabelle mit dem Namen INCLUDE existieren, würde
 * sich ein Konflikt ergeben. Aus diesem Grund kann in der Klasse
 * {@link DbConfigurationCallback} der Name der Include-Tabelle geändert werden
 * ({@link DbConfiguration#setIncludeTableName(String)}).
 */
public class DbExampleIncludeIntegrationTest extends AbstractExampleDaoTestCase {

    /**
     * Lädt initiale Testdaten, die andere Testdaten relativ zum eigenen
     * Verzeichnis inkludiert. Es ist zu beachten, dass die Testdaten in dem
     * Klassenpfad gesucht werden, sodass die referenzierten Dateien auch
     * relativ zum Klassenpfad liegen müssen.
     * <p/>
     * Folgendes Beispiel erläutert das Problem: Das Verzeichnis
     * <code>test/resources/de/conceptpeople</code> enthält folgende Dateien
     * <code>include.xml</code> und <code>innerinclude.xml</code>. Im
     * entsprechenden Verzeichnis im Klassenpfad
     * <code>target/test-classes/de/conceptpeople</code> liegt nur die Datei
     * <code>include.xml</code> (wie auch immer das passieren konnte). In jedem
     * Fall würde in diesem Fall die Datei <code>innerinclude.xml</code> nicht
     * gefunden werden, wenn sie in <code>include.xml</code> relativ
     * referenziert werden würde.
     * <p/>
     * Bei der Analyse derartige Probleme hilft die Logdatei, da dort immer der
     * absolute Pfad der eingelesenen Dateien angezeigt wird. Daher kann man
     * schnell prüfen, ob die erwartete Datei relativ dazu in der
     * Verzeichnisstruktur gefunden werden kann.
     */
    @Test
    public void testRelativeInclude() {

        assertEquals(6, getUserDao().getUsers().size());

        // Sowohl die Daten aus TheSimpsonsAdults.xml als auch...
        assertNotNull(getUserDao().getUser("Homer", "Simpson"));
        // ...die Daten aus TheSimpsonsKids.xml müssen vorhanden sein.
        assertNotNull(getUserDao().getUser("Bart", "Simpson"));

        // Außerdem müssen die normalen Testdaten vorhanden sein.
        assertNotNull(getUserDao().getUser("Santa", "Simpson"));
    }

    /**
     * Testet das Laden von initialen Testdaten, wenn eine Datei aus dem
     * Klassenpfad referenziert wurde. Das Referenzieren von Dateien aus dem
     * Klassenpfad ist z.B. sinnvoll, wenn Projektgrenzen überschritten werden.
     * Generelle Testdaten könnten in einem eigenen Basisprojekt (Maven-Projekt)
     * verwaltet werden, die dann von anderen konkreteren Projekten verwendet
     * werden. Auch das Laden von Testdaten aus JAR-Archiven ist möglich.
     */
    @Test
    public void testClasspathInclude() throws Exception {
        assertEquals(6, getUserDao().getUsers().size());

        // Sowohl die Daten aus GrownUpSimpsons.xml als auch...
        assertNotNull(getUserDao().getUser("Homer", "Simpson"));
        // ...die Daten aus KidsSimpsons.xml müssen vorhanden sein.
        assertNotNull(getUserDao().getUser("Bart", "Simpson"));

        // Außerdem müssen die normalen Testdaten vorhanden sein.
        assertNotNull(getUserDao().getUser("Santa", "Simpson"));
    }
}
