/*
 * checkerberry test center - https://www.checkerberry.de
 *
 * Copyright (c) 2010 ConceptPeople consulting gmbh
 *
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/software/page/eupl5
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.conceptpeople.checkerberry.db.example.validator;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.junit.Before;
import org.junit.ComparisonFailure;
import org.junit.Test;

import de.conceptpeople.checkerberry.common.validation.DefaultStringValidator;
import de.conceptpeople.checkerberry.common.validation.OperatorContainingDateValidator;
import de.conceptpeople.checkerberry.common.validation.OperatorContainingNumberValidator;
import de.conceptpeople.checkerberry.db.bridge.context.DatabaseTableDescription;
import de.conceptpeople.checkerberry.db.bridge.context.ValidatorContext;
import de.conceptpeople.checkerberry.db.common.exception.NoActiveValidatorException;
import de.conceptpeople.checkerberry.db.example.AbstractExampleTestCase;
import de.conceptpeople.checkerberry.db.example.ExampleConfigurationCallback;
import de.conceptpeople.checkerberry.db.example.WeightValidator;

/**
 * Beispieltest für die Verwendung von Validatoren. Checkerberry db verwendet
 * bereits Standardvalidatoren zur Überprüfung von Timestamps, Datum, Zahlen und
 * Strings. Die Auswahl des Validators erfolgt anhand der erwarteten Werte.
 * Jeder Validator prüft, ob er für die Validierung des definierten erwarteten
 * Werts zuständig ist. Am einfachsten probieren sie es aus, indem sie die
 * Testdaten so ändern, dass die Prüfung fehlschlägt. In der Fehlermeldung wird
 * dann auch der Validator angezeigt, der für die Überprüfung verwendet wurde.
 */
public class DbExampleValidatorIntegrationTest extends AbstractExampleTestCase {
    private static final Logger LOG = LoggerFactory.getLogger(DbExampleValidatorIntegrationTest.class);

    @Override
    @Before
    public void setUp() {

        super.setUp();

        // Der eigene Validator wird zunächst deaktiviert und nur in der
        // speziellen Testmethode aktiviert.
        getEnvironment().getContextHolder().getValidatorContext()
                .setValidatorActive(WeightValidator.ID, false);
    }

    /**
     * Zeigt die Verwendung des Date-Validators.
     *
     * @throws Exception
     *             wenn ein Fehler aufgetreten ist.
     */
    @Test
    public void testDateValidator() throws Exception {
        // Die erwarteten Testdaten
        // DbExampleValidatorIntegrationTest_testDateValidator_result.xml
        // enthalten erwartete Daten mit Operatoren.
        getTestHandler().assertEqualsExpected();
    }

    /**
     * Zeigt die Verwendung des Number-Validators.
     *
     * @throws Exception
     *             wenn ein Fehler aufgetreten ist.
     */
    @Test
    public void testNumberValidator() throws Exception {
        // Die erwarteten Testdaten
        // DbExampleValidatorIntegrationTest_testNumberValidator_result.xml
        // enthalten erwartete Daten mit Operatoren.
        getTestHandler().assertEqualsExpected();

        // Probieren Sie mal aus, was passiert, wenn sie in den erwarteten
        // Testdaten die Inhalte der Tabelle WEIGHT als Integer angeben, d.h.
        // z.B. "80" anstelle von "80.0". Die Prüfung wird dann fehlschlagen, da
        // der NumberValidator sich nicht mehr zuständig fühlt und die
        // Validierung an den String-Validator weiterreicht. Die Strings "80"
        // und "80.0" sind aber unterschiedlich. Probieren Sie auch aus, den
        // Wert in den erwarteten Daten von "80.0" auf "80.00" zu ändern. Das
        // funktioniert, da dann immer noch der NumberValidator zuständig ist.

    }

    /**
     * Zeigt die Konfigurationsmöglichkeiten im Umgang mit Validatoren.
     *
     * @throws Exception
     *             wenn ein Fehler aufgetreten ist.
     */
    @Test
    public void testValidatorConfiguration() throws Exception {
        // Die erwarteten Testdaten
        // DbExampleValidatorIntegrationTest_testValidatorConfiguration_result.xml
        // enthalten erwartete Daten mit Operatoren. Alle Prüfungen sind ok.
        getTestHandler().assertEqualsExpected();

        // Es besteht aber die Möglichkeit, einzelne Validatoren zu
        // deaktivieren. Wenn der Number-Validator deaktiviert wird, kommt es zu
        // Problemen in der WEIGHT-Spalte.
        ValidatorContext validatorContext = getTestHandler().getValidatorContext();
        validatorContext.setValidatorActive(OperatorContainingNumberValidator.ID, false);

        try {
            getTestHandler().assertEqualsExpected();
            fail("Hier sollten wir nicht hinkommen...");
        } catch (ComparisonFailure e) {
            LOG.info("In der Fehlermeldung sieht man, dass die Spalte WEIGHT mit dem "
                    + "String-Validator validiert wurde, da der Number-Validator nicht mehr aktiv ist.", e);
        }

        // Wenn der Validator wieder aktiviert wird, funktioniert alles wieder.
        validatorContext.setValidatorActive(OperatorContainingNumberValidator.ID, true);
        getTestHandler().assertEqualsExpected();

        // Man kann aber auch die Reihenfolge der Validatoren ändern. Per
        // Default werden die Validatoren in der folgenden Reihenfolge
        // durchlaufen: Time, Date, Number und String, wobei String alle Werte
        // validiert.
        // Ändert man jetzt die Reihenfolge um in Time, Date, String und Number,
        // verhungert der Number-Validator, da der String-Validator alle Werte
        // validiert und keinen Wert weiterreicht.
        // Validatoren, die bei der Festlegung einer neuen Reihenfolge nicht
        // erwähnt werden, werden deaktiviert. In dem Beispiel bedeutet das
        // also, dass der Time-Validator nicht mehr berücksichtigt wird.
        validatorContext.setValidatorIdsInExecutionOrder(OperatorContainingDateValidator.ID,
                DefaultStringValidator.ID, OperatorContainingNumberValidator.ID);

        try {
            getTestHandler().assertEqualsExpected();
            fail("Hier sollten wir nicht hinkommen...");
        } catch (ComparisonFailure e) {
            LOG.info("In der Fehlermeldung sieht man, dass die Spalte WEIGHT mit dem "
                    + "String-Validator validiert wurde, da der Number-Validator nicht mehr erreicht wird.",
                    e);
        }

    }

    /**
     * Zeigt die Verwendung eines eigenen Validators. Der Validator
     * {@link WeightValidator} wird in {@link ExampleConfigurationCallback}
     * registriert.
     *
     * @throws Exception
     *             wenn ein Fehler aufgetreten ist.
     */
    @Test
    public void testCustomValidator() throws Exception {

        // Der WeightValidator wird aktiviert, da er im Setup aufgrund der
        // anderen Testfälle deaktiviert wurde.
        getEnvironment().getContextHolder().getValidatorContext()
                .setValidatorActive(WeightValidator.ID, true);

        // Die erwarteten Testdaten
        // DbExampleValidatorIntegrationTest_testCustomValidator_result.xml
        // enthalten erwartete Daten, die an den WeightValidator angepasst sind.
        getTestHandler().assertEqualsExpected();
    }

    /**
     * Zeigt die Konfigurationsmöglichkeiten der Validatoren an einzelnen
     * Spalten.
     *
     * @throws Exception
     *             wenn ein Fehler aufgetreten ist.
     */
    @Test
    public void testValidatorColumnConfiguration() throws Exception {

        // Über die Tabellenbeschreibung können Validator-Einstellungen auch
        // direkt an einzelnen Spalten vorgenommen werden. Dies kann entweder
        // wie hier direkt im Test geschehen, wenn der Test sehr spezielle
        // Eigenschaften prüft. Alternativ kann das aber auch im
        // DatabaseDescriptionCallback erfolgen. Dann sind die Einstellungen für
        // alle Tests gültig.
        DatabaseTableDescription usersDescription = getTestHandler().getDatabaseDescription()
                .getTableDescription("USERS");

        // Für die Spalte WEIGHT in der Tabelle USERS wird immer der
        // String-Validator verwendet. Aufgrund der erwarteten Testdaten schlägt
        // der Vergleich fehl.
        usersDescription.setValidatorIdsInExecutionOrder("WEIGHT", DefaultStringValidator.ID);

        try {
            getTestHandler().assertEqualsExpected();
            fail("Hier sollten wir nicht hinkommen...");
        } catch (ComparisonFailure e) {
            LOG.info("In der Fehlermeldung sieht man, dass die Spalte WEIGHT mit dem "
                    + "String-Validator validiert wurde. Die Spalten sind aber für den "
                    + "WeightValidator konfiguriert.", e);
        }

        // Wenn man jetzt die Spalte WEIGHT aus dem Vergleich ausschließt,
        // funktioniert die Überprüfung der erwarteten Testdaten wieder. Man
        // sieht also, dass die Einstellung der Validator-Reihenfolge sich
        // lediglich auf die Spalte WEIGHT auswirkt. Auf diese Art und Weise
        // kann man für einzelne Spalten genau einen Validator definieren.
        usersDescription.addExcludedColumns("WEIGHT");

        // Jetzt funktioniert der Vergleich wieder.
        getTestHandler().assertEqualsExpected();

        // Wenn man die Spalte WEIGHT wieder in den Vergleich aufnimmt,
        // scheitert der Vergleich wieder.
        usersDescription.removeExcludedColumns("WEIGHT");

        try {
            getTestHandler().assertEqualsExpected();
            fail("Hier sollten wir nicht hinkommen...");
        } catch (ComparisonFailure e) {
            LOG.info("In der Fehlermeldung sieht man, dass die Spalte WEIGHT mit dem "
                    + "String-Validator validiert wurde. Die Spalten sind aber für den "
                    + "WeightValidator konfiguriert.", e);
        }

        // Durch die Aktivierung des WeightValidators und der expliziten
        // Verwendung in der Spalte WEIGHT ist der Vergleich dann wieder
        // erfolgreich.
        usersDescription.setValidatorActive(WeightValidator.ID, "WEIGHT", true);
        usersDescription.setValidatorIdsInExecutionOrder("WEIGHT", WeightValidator.ID);

        getTestHandler().assertEqualsExpected();
    }

    /**
     * Zeigt was passiert, wenn gar kein Validator aktiv ist. Das kann z.B.
     * passieren, wenn man einzelne Validatoren direkt oder indirekt durch die
     * Anpassung der Ausführungsreihenfolge deaktiviert.
     *
     * @throws Exception
     *             wenn ein Fehler aufgetreten ist.
     */
    @Test
    public void testNoActiveValidators() throws Exception {
        getTestHandler().getValidatorContext().setAllValidatorsActive(false);

        try {
            getTestHandler().assertEqualsInitial();
            fail("Hier sollten wir nicht hinkommen.");
        } catch (NoActiveValidatorException e) {
            LOG.info("Fehlermeldung, dass kein Validator aktiv ist.", e);
        }
    }
}
